/*
 * Copyright (c) 2003
 * Gerasimos Dimitriadis, Aristotle University of Thessaloniki.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GERASIMOS DIMITRIADIS AND THE ARISTOTLE
 * UNIVERSITY OF THESSALONIKI ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * SHALL GERASIMOS DIMITRIADIS OR THE ARISTOTLE UNIVERSITY OF THESSALONIKI
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef _NETGRAPH_DAPHNE_H_
#define _NETGRAPH_DAPHNE_H_

/* Node type name and magic cookie */
#define NG_DAPHNE_NODE_TYPE	"daphne"
#define NGM_DAPHNE_COOKIE	310996380

/* Hook names */
#define NG_DAPHNE_HOOK_DOWNSTREAM	"downstream"
#define NG_DAPHNE_HOOK_UPSTREAM		"upstream"
#define NG_DAPHNE_HOOK_INFO			"info"
#define NG_DAPHNE_HOOK_PASS			"pass"

/* Operating Parameters */
#define MAX_SPARE_ROWHEADS		10
#define MIN_SPARE_ROWHEADS		5

#define MAX_SPARE_RTELEMS		20
#define MIN_SPARE_RTELEMS		10

#define BEACON_PERIOD			1
#define BEACON_MISSES_TILL_TO	5
#define BEACON_ROUTING_RATIO	5
#define TICKS_UNTIL_ROUTING_TO	20
#define TICKS_UNTIL_DATA_TO		30

#define DAPHNE_ETHER_TYPE		0x4444
#define DAPHNE_TTL				15

#define MAX_UPDATE_SIZE			1500
#define DAPHNE_INFINITY			255

/* Message types */
#define DAPHNE_BEACON			1
#define DAPHNE_ROUTING			2
#define DAPHNE_DATA				3

/* Info Types */
#define INFO_ROUTING_TABLE		1

/* Useful structures */
struct rtElem
{
	u_int8_t		 etherAddr[6];
	u_int8_t		 timeOut;
	struct rtElem	*next;
};

struct rowHead
{
	u_int8_t		 etherAddr[6];
	u_int8_t		 numNeighb;
	struct rtElem	*neighbours;
	u_int8_t		 nextHop[6];
	u_int8_t		 numHops;
	u_int32_t		 dataSeqNum;
	u_int8_t		 dataSeqNumTimer;
	u_int32_t		 routingSeqNum;
	u_int8_t		 routingSeqNumTimer;
	struct rowHead	*next;
};

/* Header used for encapsulation */
struct daphne_header {
	u_char  type;
	u_char  ttl;
	u_short size;
	u_long  seq_num;
	u_char  ether_dhost[6];
	u_char  ether_shost[6];
	u_short orig_type;
} __attribute__ ((packed));

#define DAPHNE_HDR_LEN sizeof (struct daphne_header)

/* Netgraph control messages */
enum {
	NGM_DAPHNE_SET_ENADDR = 1,
	NGM_DAPHNE_GET_ENADDR,
};

#endif

