/*
 * Copyright (c) 2003
 * Gerasimos Dimitriadis, Aristotle University of Thessaloniki.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GERASIMOS DIMITRIADIS AND THE ARISTOTLE
 * UNIVERSITY OF THESSALONIKI ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * SHALL GERASIMOS DIMITRIADIS OR THE ARISTOTLE UNIVERSITY OF THESSALONIKI
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef NG_DAPHNE_MACROS_H

#define NG_DAPHNE_MACROS_H

#include "ng_daphne.h"

#define ETHER_EQUAL(a,b)	(((const u_int32_t *)(a))[0] 						\
		== ((const u_int32_t *)(b))[0] 											\
		&& ((const u_int16_t *)(a))[2] 											\
		== ((const u_int16_t *)(b))[2])

#define FREE_CHAIN(head, type)	do {											\
		struct type *temp, *temp2;												\
		if (head != NULL)														\
		{																		\
			temp = head;														\
			do																	\
			{																	\
				temp2 = temp->next;												\
				FREE (temp, M_NETGRAPH);										\
				temp = temp2;													\
			} while (temp != NULL);												\
		}																		\
	} while (0)

#define ADD_TO_CHAIN(head_p, elem_p, counter)	do {							\
		elem_p->next = NULL;													\
		if (head_p == NULL)														\
		{																		\
			head_p = elem_p;													\
			counter = 1;														\
		}																		\
		else																	\
		{																		\
			elem_p->next = head_p;												\
			head_p = elem_p;													\
			counter ++;															\
		}																		\
	} while (0)

#define DEQUEUE_ELEM(head_p, elem_p, counter)	do {							\
		if (head_p != NULL)														\
		{																		\
			elem_p = head_p;													\
			head_p = head_p->next;												\
			elem_p->next = NULL;												\
			counter--;															\
		}																		\
		else																	\
		{																		\
			elem_p = NULL;														\
		}																		\
	} while (0)

#define FIND_ROWHEAD(result, addr)	do {										\
		struct rowHead *temp;													\
		result = NULL;															\
		for (temp = priv->routingTable; temp != NULL; temp = temp->next)		\
		{																		\
			if (ETHER_EQUAL(temp->etherAddr, addr))								\
			{																	\
				result = temp;													\
				break;															\
			}																	\
		}																		\
	} while (0)



#define SEQ_GT(a,b)		((long int)((a)-(b)) > 0)

#endif
